==> viqcore_flutter-1.2.2.tar.gz: lib/src/http_client/interceptors.dart <==
import 'package:flutter/material.dart';

import '../app.dart';
import 'utils/dio_helper.dart';
import 'utils/nullable_string_helper.dart';

class ApiInterceptor extends Interceptor {
  @override
  void onRequest(
      RequestOptions options, RequestInterceptorHandler handler) async {
    if (!options.isGQL) {
      options.extra['__data__'] = options.data;
      options.data = await _encodeBody(options.data);
      options.queryParameters = _encodeQuery(options.queryParameters);
    }

    if (!options.isExternal) {
      options.headers.addAll({
        'timezoneoffset': '${DateTime.now().timeZoneOffset.inHours}',
        if (App<API>().apiKey.isNotNullOrEmpty) 'apikey': App<API>().apiKey,
      });
    }
    handler.next(options);
  }

  @override
  void onResponse(Response response, ResponseInterceptorHandler handler) async {
    if (response.requestOptions.isGQL) {
      if (response.gqlErrors) {
        handler.reject(
            DioException(
              requestOptions: response.requestOptions,
              error: response.gqlErrorMessage,
              message: response.gqlErrorMessage,
              response: response,
            ),
            response.isUnauthorized);
        return;
      }
      final data = response.data['data'] as Map<String, dynamic>;
      response.data = data.keys.length == 1 ? data.values.first : data;
    }

    if (!response.requestOptions.isExternal) {
      final refreshToken = response.headers.value('x-refresh-token');
      if (refreshToken.isNotNullOrEmpty) {
        App.passport.setRefreshToken(refreshToken);
      }
    }

    handler.next(response);
  }

  @override
  void onError(DioException err, ErrorInterceptorHandler handler) async {
    if (err.requestOptions.isRenewCredential) {
      if (!err.isNetworkError) {
        App.passport.signedOut();
      }
    } else if (err.response != null && err.response!.isUnauthorized) {
      if (!err.requestOptions.isRetry) {
        debugPrint(
            'RenewAuthToken by: ${err.requestOptions.isGQL ? err.requestOptions.data : err.requestOptions.path}');
        await renewAuthToken();
        if (App.passport.refreshToken.isNullOrEmpty) {
          handler.reject(err);
        } else {
          // retry again
          final requestOptions = err.requestOptions;

          try {
            requestOptions.setRetryFlag();
            final response = await App.getIt<API>().dio.request(
                requestOptions.path,
                data: requestOptions.extra['__data__'] ?? requestOptions.data,
                queryParameters: requestOptions.queryParameters,
                options: Options(
                  method: requestOptions.method,
                  extra: requestOptions.extra,
                  headers: requestOptions.headers,
                ));
            if (response.gqlErrors) {
              handler.reject(DioException(
                requestOptions: response.requestOptions,
                error: response.gqlErrorMessage,
                message: response.gqlErrorMessage,
                response: response,
              ));
              App.passport.onFailedAuthorizationFromServer();
            } else {
              handler.resolve(response);
            }
          } on DioException catch (retryErr) {
            handler.reject(retryErr);
            App.passport.onFailedAuthorizationFromServer();
          } on Exception catch (_) {}
        }
      }
    }
    if (!handler.isCompleted) handler.next(err);
  }

  Future renewAuthToken() async {
    if (App.passport.refreshToken.isNullOrEmpty) return;
    return App.passport.refreshTokenFromServer();
  }
}

Future<dynamic> _encodeBody(dynamic body) async {
  if (body == null) return null;
  if (body is RestBody) {
    final encoded = await _encodeJson(body.data);
    if (body.formDataMode) {
      return FormData.fromMap(encoded);
    }
    return encoded;
  }
  final encodedBody = await _encodeJson(body);
  if (encodedBody == null) return null;
  if (encodedBody is Map<String, dynamic> &&
      encodedBody.values.any((element) =>
          element is MultipartFile || element is List<MultipartFile>)) {
    return FormData.fromMap(encodedBody);
  }
  return encodedBody;
}

// https://stackoverflow.com/questions/72634402/unhandled-exception-bad-state-cant-finalize-a-finalized-multipartfile-and-onl
Future<dynamic> _encodeJson(dynamic payload) async {
  if (payload == null) return null;
  if (payload is! Map<String, dynamic>) return payload;

  final encoded = Map<String, dynamic>.from(payload)
    ..removeWhere((_, v) => v == null);

  for (final key in encoded.keys.toList()) {
    final value = encoded[key];

    if (value is List<DateTime>) {
      encoded[key] = value.map((e) => e.toIso8601String()).toList();
    } else if (value is DateTime) {
      encoded[key] = value.toIso8601String();
    } else if (value is XFile) {
      encoded[key] = MultipartFile.fromBytes(
        await value.readAsBytes(),
        filename: value.name,
      );
    } else if (value is List<XFile>) {
      encoded[key] = await Future.wait(value.map((file) async {
        final bytes = await file.readAsBytes();
        return MultipartFile.fromBytes(
          bytes,
          filename: file.name,
        );
      }));
    }
  }

  return encoded;
}

Map<String, dynamic> _encodeQuery(Map<String, dynamic> query) {
  return query
    ..removeWhere((key, value) => value == null)
    ..map((key, value) {
      if (value is List) {
        return MapEntry(key, value.map((e) => e.toString()).toList());
      }
      return MapEntry(key, value.toString());
    });
}

==> viqcore_flutter-1.3.1.tar.gz: lib/src/http_client/interceptors.dart <==
import 'package:flutter/material.dart';

import '../app.dart';
import 'utils/dio_helper.dart';
import 'utils/nullable_string_helper.dart';

class ApiInterceptor extends Interceptor {
  @override
  void onRequest(
      RequestOptions options, RequestInterceptorHandler handler) async {
    if (!options.isGQL) {
      options.extra['__data__'] = options.data;
      options.data = await _encodeBody(options.data);
      options.queryParameters = _encodeQuery(options.queryParameters);
    }

    if (!options.isExternal) {
      options.headers.addAll({
        'timezoneoffset': '${DateTime.now().timeZoneOffset.inHours}',
        if (App<API>().noCookie) 'no-cookie': true,
        if (App<API>().apiKey.isNotNullOrEmpty) 'apikey': App<API>().apiKey,
        if (App.passport.accessToken.isNotNullOrEmpty)
          'Authorization': 'Bearer ${App.passport.accessToken}',
      });
    }
    handler.next(options);
  }

  @override
  void onResponse(Response response, ResponseInterceptorHandler handler) async {
    if (response.requestOptions.isGQL) {
      if (response.gqlErrors) {
        handler.reject(
            DioException(
              requestOptions: response.requestOptions,
              error: response.gqlErrorMessage,
              message: response.gqlErrorMessage,
              response: response,
            ),
            response.isUnauthorized);
        return;
      }
      final data = response.data['data'] as Map<String, dynamic>;
      response.data = data.keys.length == 1 ? data.values.first : data;
    }

    if (!response.requestOptions.isExternal) {
      final logoutToken = response.headers.value("x-logout");
      if (logoutToken.isNotNullOrEmpty) {
        App.passport.signedOut();
      }
      final accessToken = response.headers.value('x-access-token');
      if (accessToken.isNotNullOrEmpty) {
        App.passport.setAccessToken(accessToken!);
      }
      final refreshToken = response.headers.value('x-refresh-token');
      if (refreshToken.isNotNullOrEmpty) {
        App.passport.setRefreshToken(refreshToken!);
      }
    }

    handler.next(response);
  }

  @override
  void onError(DioException err, ErrorInterceptorHandler handler) async {
    if (err.requestOptions.isRenewCredential) {
      if (!err.isNetworkError) {
        App.passport.signedOut();
      }
    } else if (err.response != null && err.response!.isUnauthorized) {
      if (!err.requestOptions.isRetry) {
        debugPrint(
            'RenewAuthToken by: ${err.requestOptions.isGQL ? err.requestOptions.data : err.requestOptions.path}');
        await renewAuthToken();
        if (App.passport.refreshToken.isNullOrEmpty) {
          handler.reject(err);
        } else {
          // retry again
          final requestOptions = err.requestOptions;

          try {
            requestOptions.setRetryFlag();
            final response = await App.getIt<API>().dio.request(
                requestOptions.path,
                data: requestOptions.extra['__data__'] ?? requestOptions.data,
                queryParameters: requestOptions.queryParameters,
                options: Options(
                  method: requestOptions.method,
                  extra: requestOptions.extra,
                  headers: requestOptions.headers,
                ));
            if (response.gqlErrors) {
              handler.reject(DioException(
                requestOptions: response.requestOptions,
                error: response.gqlErrorMessage,
                message: response.gqlErrorMessage,
                response: response,
              ));
              App.passport.onFailedAuthorizationFromServer();
            } else {
              handler.resolve(response);
            }
          } on DioException catch (retryErr) {
            handler.reject(retryErr);
            App.passport.onFailedAuthorizationFromServer();
          } on Exception catch (_) {}
        }
      }
    }
    if (!handler.isCompleted) handler.next(err);
  }

  Future renewAuthToken() async {
    if (App.passport.refreshToken.isNullOrEmpty) return;
    return App.passport.refreshTokenFromServer();
  }
}

Future<dynamic> _encodeBody(dynamic body) async {
  if (body == null) return null;
  if (body is RestBody) {
    final encoded = await _encodeJson(body.data);
    if (body.formDataMode) {
      return FormData.fromMap(encoded);
    }
    return encoded;
  }
  final encodedBody = await _encodeJson(body);
  if (encodedBody == null) return null;
  if (encodedBody is Map<String, dynamic> &&
      encodedBody.values.any((element) =>
          element is MultipartFile || element is List<MultipartFile>)) {
    return FormData.fromMap(encodedBody);
  }
  return encodedBody;
}

// https://stackoverflow.com/questions/72634402/unhandled-exception-bad-state-cant-finalize-a-finalized-multipartfile-and-onl
Future<dynamic> _encodeJson(dynamic payload) async {
  if (payload == null) return null;
  if (payload is! Map<String, dynamic>) return payload;

  final encoded = Map<String, dynamic>.from(payload)
    ..removeWhere((_, v) => v == null);

  for (final key in encoded.keys.toList()) {
    final value = encoded[key];

    if (value is List<DateTime>) {
      encoded[key] = value.map((e) => e.toIso8601String()).toList();
    } else if (value is DateTime) {
      encoded[key] = value.toIso8601String();
    } else if (value is XFile) {
      encoded[key] = MultipartFile.fromBytes(
        await value.readAsBytes(),
        filename: value.name,
      );
    } else if (value is List<XFile>) {
      encoded[key] = await Future.wait(value.map((file) async {
        final bytes = await file.readAsBytes();
        return MultipartFile.fromBytes(
          bytes,
          filename: file.name,
        );
      }));
    }
  }

  return encoded;
}

Map<String, dynamic> _encodeQuery(Map<String, dynamic> query) {
  return query
    ..removeWhere((key, value) => value == null)
    ..map((key, value) {
      if (value is List) {
        return MapEntry(key, value.map((e) => e.toString()).toList());
      }
      return MapEntry(key, value.toString());
    });
}
